import React, { Component } from 'react';
import Icon from 'react-edp-icons';
import { EDP_DROPDOWN_MENU, EDP_DROPDOWN_MENU_ARROW_RIGHT, EDP_SUBMENU } from './css-class-token'
import '../styles/EDP-dropdown-menu.css';

import { Submenu } from './DropDown/Submenu';
import { eventManager } from './DropDown/utils/eventManager';
import { stringCleaner } from '../util'

export default class DropDown extends Component {
    constructor() {
        super();
        this.state = {
            clientX: 0,
            clientY: 0,
            ctx: []
        };
        this.showSubMenu = this.showSubMenu.bind(this);
        this.hiddenSubMenu = this.hiddenSubMenu.bind(this);
        this.suBmenuRef = React.createRef();
        this.setWrapperRef = this.setWrapperRef.bind(this);
        this.handleClickOutside = this.handleClickOutside.bind(this);
    }

    componentDidMount() {
        document.addEventListener('mousedown', this.handleClickOutside);

        
    }

    componentWillUnmount() {
        document.removeEventListener('mousedown', this.handleClickOutside);
       
    }

    showSubMenu(event, ctx) {
        event.stopPropagation();
        event.nativeEvent.stopImmediatePropagation();

        const suBmenuPosition = document
            .getElementById(`${ctx.contexMenuId}`)
            .getBoundingClientRect();
        this.suBmenuRef = document.getElementById(`${ctx.contexMenuId}`);
        this.suBmenuRef.style.background = 'rgba(0, 0, 0, 0.1)';
        const propsToDisplay = {
            clientX: suBmenuPosition.x + 200,
            clientY: suBmenuPosition.y,
            ctx,
            visible: 'visible',
            suBmenuId: `${ctx.contexMenuId}`,
            event
        };
        eventManager.emit(`DISPLAY_${this.suBmenuRef.id}`, propsToDisplay);
    }

    hiddenSubMenu() {
        event.stopPropagation();
        this.suBmenuRef.style.background = '#fff';
        Array.from(document.querySelectorAll(`.${EDP_SUBMENU}`)).map(ele => ele.style.visibility = 'hidden');
    }

    hideMenu() {
        event.stopPropagation();
        Array.from(document.querySelectorAll(`.${EDP_DROPDOWN_MENU}`)).map(ele => (ele.style.visibility = 'hidden'));
    }

    setWrapperRef(node) {
        this.wrapperRef = node;
    }

    handleClickOutside(event) {
        if (this.wrapperRef && !this.wrapperRef.contains(event.target)) {
            this.hideMenu();
        }
    }

    handleSubMenuClick(event, ctx, row) {
        this.props.handleContextClick(ctx, row, true);
    }

    render() {
        const { row, handleContextClick } = this.props;
        const iconEnum = {
            EditWithNativeApplication: '',
            EDPLaunchForwardWFAction: 'ENCAMINHAR',
            EDPlaunchExtDivulgationWFAction: 'DIVULGAR',
            EDPAssociateDocsPluginAction: 'ASSOCIAR_DOCUMENTO',
            Edit: 'PROPRIEDADEs',
            EDPDocumentcontextmenu_L$0: 'EDITAR_PROPRIEDADE',
            EDPDocumentcontextmenu_L$1: 'MINHAS_PASTAS',
            EDPlaunchExternalWFRepubAction: 'REPUBLICAR',
            ShowHyperlink: 'LINK_DOCUMENTO',
            EDPDownloadStampedPDFAction: 'DESCARREGAR',
            EDPPrintStampedPDFAction: 'DESCARREGAR',
            LaunchWorkflowAction: '',
            ViewChildDocuments: '',
            EDPOpenProcessAction: 'PROCESSO'
        };
        return (
			<React.Fragment>
				<span className="TreeDots__hover"/>
				<div className={`${EDP_DROPDOWN_MENU}`} ref={this.setWrapperRef} id={stringCleaner(row.rowInfos.id)}>
					<div className="wrapper wrapper-loading">
						<span className="Dots__menu"/>
						<ul>
							{row.contexMenu.map(ctx => {
                                let iconToRender = iconEnum[ctx.id] ? iconEnum[ctx.id] : 'DESCARREGAR';

								return ctx.subActions.length
									? <li
											onClick={event => this.handleSubMenuClick(event, ctx, row)}
											ref={this.suBmenuRef}
											onMouseEnter={event => this.showSubMenu(event, ctx)}
											onMouseLeave={event => this.hiddenSubMenu(event)}
											id={`${ctx.contexMenuId}`}
										>
											<div className="icon-context-menu">
												<Icon  icon={iconToRender} size={30} />
											</div>
											<div className="name-context-menu">
												{ctx.name}
											</div>
											<div className={`${EDP_DROPDOWN_MENU_ARROW_RIGHT}`}>
												<Icon icon={'ARROW_RIGHT'} size={15} />
											</div>
											<Submenu
												item={ctx.subActions}
												suBmenuId={`${ctx.contexMenuId}`}
												style={{
													clientY: this.state.clientY,
													clientX: this.state.clientX,
													visible: this.state.visible
												}}
												idMenuToShow={this.state.suBmenuId}
                                                rowId={row.rowInfos.id}
                                                rowItem={row.rowInfos}
												handleClick={ctx => handleContextClick(ctx, row)}
											/>
										</li>
									: <li onClick={event => handleContextClick(ctx, row)} id={`${ctx.contexMenuId}`}>
											<div className="icon-context-menu">
												<Icon icon={iconToRender} size={30} />
											</div>
											<div className="name-context-menu">
												{ctx.name}
											</div>
										</li>;
							})}
						</ul>
					</div>
				</div>
			</React.Fragment>
		);
	}
}
