import React from 'react';
import { createPortal } from 'react-dom';
import { eventManager } from './utils/eventManager';

import { EDP_SUBMENU } from '../css-class-token'
import { stringCleaner, getFullAttributes } from '../../util';

class Portal extends React.Component {
    constructor() {
        super();
        this.state = {
            container: document.createElement('div'),
            showSubMenu:false
        };
        this.show = this.show.bind(this);
        this.setContainerId = this.setContainerId.bind(this);
        this.unsub = [];
    }

    componentWillUnmount() {
        this.unsub.forEach(cb => cb());
    }

    componentDidUpdate() {
        const {
            item,
            rowId,
            rowItem
        } = this.props;
        if(this.state.showSubMenu){
            const defaultRepository = ecm.model.desktop.getDefaultRepository();
            item.forEach(async subAction => {
                const subMenuId  = `${stringCleaner(subAction.name)}___${stringCleaner(rowId)}_${EDP_SUBMENU}`;
                const subMenuEl = document.getElementById(subMenuId);
                if (!subMenuEl) return;
                const fullItem = await getFullAttributes(rowItem);
                subAction.isEnabled(defaultRepository, '', [fullItem], null, null) ?  subMenuEl.className = '': subMenuEl.className = 'disabled';
                subAction.isVisible(defaultRepository, [fullItem], null, null) ? subMenuEl.style.display = 'flex' : subMenuEl.style.display = 'none';
            });
        }
    }


    show(eventProps) {
        const { clientX, clientY, suBmenuId, event  } = eventProps;
        event.stopPropagation();
        event.nativeEvent.stopImmediatePropagation();
        const ele = document.querySelectorAll(`#${suBmenuId}_${EDP_SUBMENU}`)[0];
        const { innerWidth, innerHeight } = window;
        if (ele) {
            const subMenuStyle = {
                top: `${clientY}px`,
                left: `${clientX}px`,
                visibility: 'visible'
            }
            if (innerWidth > clientX) {

                subMenuStyle.left = `${clientX - 480}px`
            }

            if (innerHeight > clientY) {
              
                subMenuStyle.top = `${clientY}px`
            }

            const menuStyleIntoString = JSON.stringify(subMenuStyle).replace(/[{}""]/g, '').split(' ').join('').replace(/,/g, '; ')
            ele.setAttribute('style', menuStyleIntoString)
        } else {
            Array.from(document.querySelectorAll(`${EDP_SUBMENU}`))
                .filter(ele => ele.id !== suBmenuId)
                .map(ele => (ele.style.visibility = 'hidden'));
        }
        this.setState({
            container: ele,
            showSubMenu:true
        });
    }

    setContainerId() {
        const container = this.state.container;
        container.setAttribute('id', `${this.props.suBmenuId}_${EDP_SUBMENU}`);
        if (document.getElementById(`${this.props.suBmenuId}_${EDP_SUBMENU}`))
            return;
        document.body.appendChild(container);
        this.setState({
            canRender: true,
            container
        });

    }

    componentDidMount() {
        this.unsub.push(eventManager.on(`DISPLAY_${this.props.suBmenuId}`, this.show));
        const container = document.createElement('div');
        container.className = `${EDP_SUBMENU}`;
        container.setAttribute('id', `${this.props.suBmenuId}_${EDP_SUBMENU}`);
        this.setState({
            canRender: true,
            container
        });
        if (document.getElementById(`${this.props.suBmenuId}_${EDP_SUBMENU}`))
            return;
        document.body.appendChild(container);
        eventManager.on(`DISPLAY_${this.props.suBmenuId}`, this.show);
    }

    render() {
        const { container } = this.state;
   
        if (!container) {
            const container = document.createElement('div');
            return createPortal(this.props.children, container);
        }
        return createPortal(this.props.children, container);
    }
}

export { Portal };
