import React, { Component } from 'react';

import '../styles/react-table.css';
import '../styles/grid.css';
import Icon from 'react-edp-icons';

import BootstrapTable from 'react-bootstrap-table-next';
import paginationFactory from 'react-bootstrap-table2-paginator';

import DropDown from './DropDown'
import FormatWithIcon from './FormatWithIcon'
import TransformCell from './TransformCell'

import { stringCleaner, isEmpty, getFullAttributes } from '../util'
import { EDP_DROPDOWN_MENU } from './css-class-token'
import cloneDeep from 'lodash.clonedeep';
export class Grid extends Component {
  constructor(props) {
    super(props);
    this.state = {
      body: [],
      column: [],
      data: [],
      row: [],
      selected: []
    };
    this.handleTableChange = this.handleTableChange.bind(this);
    this.handleContextClick = this.handleContextClick.bind(this);
    this.handleOnSelect = this.handleOnSelect.bind(this);
    this.callAction = this.callAction.bind(this);
    this.bindWindowEvent = this.bindWindowEvent.bind(this);
    this.unBindWindowEvent = this.unBindWindowEvent.bind(this);
    this.hideDropDownMenu = this.hideDropDownMenu.bind(this);
    this.onContextMenu = this.onContextMenu.bind(this);
  }

  componentDidMount() {
    if (!this.state.column.length && !isEmpty(this.props.resultSet)) {
      const { items, structure } = this.props.resultSet;
      const { columns, data } = this.formatTable(
        structure,
        items
      );
      this.setState({ columns, data });
    }
  }

  componentDidUpdate(prevProps) {
    if (!Object.is(this.props.resultSet, prevProps.resultSet) ||
      !Object.is(this.props.additionalColumns, prevProps.additionalColumns)
    ) {
      const { items, structure } = this.props.resultSet;
      const { columns, data } = this.formatTable(
        structure,
        items
      );
      this.setState({ columns, data });
    }

    if (this.props.cleanSelect !== prevProps.cleanSelect) {
      if (this.props.cleanSelect) {
        this.setState({ selected: [] });
      }
    }
  }

  callAction(event, row) {
    event.preventDefault();
    const { isLink, overwriteMethod } = this.props;
    if (isLink) {
      overwriteMethod(row);
    } else {
      const item = row.rowInfos;
      if(item.isInstanceOf(ecm.model.ContentItem)){
          if(item.hasPrivilege('privMajorVersion')){
            this.openAction(item,'EDPEditDocumentAction');
          }else{
            this.openAction(item,'EDPPreviewDocumentAction');
          }
      }else{
          this.handleContextClick(this.props.contextMenuItems[0], row);
      }
    }
  }


  openAction(item, actionName) {
    item.repository = ecm.model.desktop.getDefaultRepository();
    const preview = ecm.model.desktop.actionLists.ItemContextMenu.find(({ id }) => id === actionName);
    getFullAttributes(item).then(itemWithAllAtt => {
      const repository = ecm.model.desktop.getDefaultRepository();
      preview.performAction(repository, [itemWithAllAtt]);
    });
  }

  formatTable(column, data) {
    if (column.cells[0].length && data) {
      const { items, structure } = this.props.resultSet;
      const columnsSemIcones = Array.from(
        structure.cells[0].filter(cel => !cel.decorator)
      ).map(this.handleColumnsSemIcones());

      const { additionalColumns, hideMimeType } = this.props;
      const icons = Array.from(
        structure.cells[0].filter(
          cel => cel.decorator && cel.field !== 'multiStateIcon'
        )
      ).map(this.handleColumnWithIcons(hideMimeType));

      const dropDownCell = {
        dataField: 'contextMenu',
        text: '',
        events: {
          onClick: (e, column, columnIndex, row, rowIndex) => {
            if (this.props.contextMenuItems) {
              this.onContextMenu(event, row)
            }
          },
        },
        formatter: (cell, row) => <DropDown row={row}  handleContextClick={this.handleContextClick} />,
        classes: 'DropDown__menu'
      }

      // Copia colunas sem icones e com icones para a renderizacao da tabela
      let columns = [...icons, ...columnsSemIcones, dropDownCell];


      if(Array.isArray(additionalColumns) && additionalColumns.length > 0) {
        columns = [...columns, ...additionalColumns];
      }
      const data = Array.from(items).map(e => {
        return {
          ...e.attributes,
          rowInfos: e,
          Id:e.id,
          mimetype: e.mimetype,
          classes: 'context-menu',
          style: { width: '20px' }
        };
      });


      data.forEach(row => {
        const contextMenu = this.props.contextMenuItems ? this.props.contextMenuItems : [];
        const contextMenuCopy = this.copyOfContextMenu(contextMenu);
        row.contexMenu = contextMenuCopy.map(action => {
          action.contexMenuId = `${stringCleaner(action.name)}___${stringCleaner(row.rowInfos.id)}`
          return action;
        });
        return row;
      });

      return { columns, data };
    }
  }

  copyOfContextMenu(contextMenu) {
    const actionToFilter = ['EDPLaunchWFSGDAction', 'ViewChildDocuments', 'Preview'];
    const filtered = contextMenu.map(action => actionToFilter.find(stg => action.id == stg) ? null : action).filter(x => x);
    return cloneDeep(filtered);
  }

  handleColumnWithIcons(hideMimeType) {
    return icon => {
      return {
        dataField: icon.field,
        text: '',
        infoIcon: icon,
        formatter: (cell, row) => <FormatWithIcon hideMimeType={hideMimeType} row={row} />,
        headerStyle: {
          width: '10px'
        }
      };
    };
  }

  handleColumnsSemIcones() {
    return e => {
      return {
        dataField: e.field,
        text: e.name,
        sort: true,
        formatter: (cell, row) => <TransformCell cell={cell} row={row} field={e.field} callAction={(event, row) => this.callAction(event, row)} />,
        sortCaret: (order, column) => {
          if (!order)
            return <Icon icon='UP' color={'#EE162D'} size={10} />;
          else if (order === 'desc')
            return <Icon icon='DOWN' color={'#333333'} size={10} />;
          else if (order === 'asc')
            return <Icon icon='UP' color={'#333333'} size={10} />;
          return null;
        }
      };
    };
  }

  handleTableChange(type, { sortField, sortOrder, data }) {
    let result;
    const getTime = date => new Date(date).getTime();
    if (sortOrder === 'asc') {
      if (/Date|Data|Time/i.test(sortField)) {
        result = data.sort((a, b) => {
          return getTime(a[sortField]) > getTime(b[sortField]) ? 1 : getTime(b[sortField]) > getTime(a[sortField]) ? -1 : 0
        });
      } else {
        result = data.sort((a, b) =>
          a[sortField] > b[sortField] ? 1 : b[sortField] > a[sortField] ? -1 : 0
        );
      }
    } else {
      if (/Date|Data|Time/i.test(sortField)) {
        result = data.sort((a, b) => {
          return getTime(a[sortField]) > getTime(b[sortField]) ? -1 : getTime(b[sortField]) > getTime(a[sortField]) ? 1 : 0
        });
      } else {
        result = data.sort((a, b) =>
          a[sortField] > b[sortField] ? -1 : b[sortField] > a[sortField] ? 1 : 0
        );
      }
    }
    this.setState({ data: result });
  }

  handleContextClick(ctx, row, isSubmenu = false) {
    this.hideDropDownMenu(isSubmenu);
    const item = row.rowInfos;
    const action = ctx;
    if (isSubmenu) return;
    getFullAttributes(item).then(itemWithAllAtt => {
      const repo = ecm.model.desktop.getDefaultRepository();
      action.performAction(repo, [itemWithAllAtt]);
    });
  }

  handleOnSelect(row, isSelect) {
    if (isSelect) {
      this.setState({ selected: [row.Id] });
      this.props.onRowSelect({ row });
    } else {
      this.setState(() => ({
        selected: this.state.selected.filter(x => x !== row.Id)
      }));
    }
  }

  bindWindowEvent() {
    window.addEventListener('resize', this.hideDropDownMenu, true);
    window.addEventListener('contextmenu', this.hideDropDownMenu, true);
    window.addEventListener('click', this.hideDropDownMenu, true);
  };

  unBindWindowEvent() {
    window.removeEventListener('resize', this.hideDropDownMenu, true);
    window.removeEventListener('contextmenu', this.hideDropDownMenu, true);
    window.removeEventListener('click', this.hideDropDownMenu, true);
  };

  hideDropDownMenu(isSubmenu = false) {
    if (typeof isSubmenu === 'boolean' && isSubmenu) return;
    this.unBindWindowEvent();
    document.querySelectorAll(`.${EDP_DROPDOWN_MENU}`).forEach(ele => ele.style.visibility = 'hidden');
    document.querySelectorAll('.react-bootstrap-table table tbody tr').forEach(el => el.style.background = '#fff');
    if (document.querySelector('.DropDown__menu.__active')) {
      const activeDropDown = document.querySelector('.DropDown__menu.__active');
      activeDropDown.classList.remove('__active')
    }
  };

  isElEnabled(ele, id) {
    const ctxLine = ele.find(({ contexMenuId }) => contexMenuId === id)
    return ctxLine.enabled
  }

  isElVisible(ele, id) {
    const ctxLine = ele.find(({ contexMenuId }) => contexMenuId === id)
    return ctxLine.visible
  }


  onContextMenu(event, row) {
    if (!this.props.contextMenuItems) return;
    const rowElement = document.querySelectorAll(`.row-class-${stringCleaner(row.rowInfos.id)}`)[0];
    rowElement.style.backgroundColor = '#F2F2F2';
    const dropDownMenu = document.getElementById(`${stringCleaner(row.rowInfos.id)}`);
    dropDownMenu.style.visibility = 'visible';

    if (document.querySelectorAll(`.row-class-${stringCleaner(row.rowInfos.id)} .DropDown__menu`)[0]) {
      const dropDownCell = document.querySelectorAll(`.row-class-${stringCleaner(row.rowInfos.id)} .DropDown__menu`)[0];
      dropDownCell.classList.add('__active');
    }

    this.loadFullContextMenu(row).then(isFinito => {
      const dropDownMenuList = document.querySelectorAll(`#${stringCleaner(row.rowInfos.id)} ul li`);
      const dropDownWrapper = document.querySelectorAll(`#${stringCleaner(row.rowInfos.id)} .wrapper.wrapper-loading`);
      
      Array.from(dropDownMenuList).forEach(listEle => {
        if (listEle) {
          listEle.className = this.isElEnabled(isFinito.contexMenu, listEle.id) ? '' : 'disabled';
          listEle.style.display = this.isElVisible(isFinito.contexMenu, listEle.id) ? 'flex' : 'none';
        }
      })

      dropDownWrapper.forEach(list => list.classList.remove('wrapper-loading'))

      this.bindWindowEvent();
    })
  }

  loadFullContextMenu(row) {
    return new Promise(resolve => {
      const defaultRepository = ecm.model.desktop.getDefaultRepository();

      getFullAttributes(row.rowInfos).then(fullItem => {
        row.contexMenu = cloneDeep(row.contexMenu).map(action => {
          const disabledSubActions = {}
          if (action.subActions.length) {
            action.subActions.map(subAction => {
              subAction.enabled = subAction.isEnabled(defaultRepository, '', [fullItem], null, null);
              subAction.visible = subAction.isVisible(defaultRepository, [fullItem], null, null);
            })
            disabledSubActions.actionId = action.contexMenuId
            disabledSubActions.disabled = action.subActions.filter(({ enabled }) => !enabled)
          }
          action.enabled = action.isEnabled(defaultRepository, '', [fullItem], null, null);
          action.visible = action.isVisible(defaultRepository, [fullItem], null, null);

          if (action.subActions.length) {
            action.enabled = action.contexMenuId === disabledSubActions.actionId && action.subActions.length === disabledSubActions.disabled.length ? false : true;
          }
          return action;
        });
        resolve(row)
      });
    })
  }

  addDotsToRow(rowId) {
    const hoverDots = document.querySelectorAll(`.row-class-${stringCleaner(rowId)} .TreeDots__hover`)[0]
    hoverDots.classList.add('__show')
  }

  removeDotsToRow(rowId) {
    const hoverDots = document.querySelectorAll(`.row-class-${stringCleaner(rowId)} .TreeDots__hover`)[0]
    hoverDots.classList.remove('__show')
  }

  render() {
    
    const pageButtonRenderer = ({
      page,
      active,
      disable,
      title,
      onPageChange
    }) => {
      const handleClick = e => {
        e.preventDefault();
        onPageChange(page);
      };
      let activeStyle = {};
      if (active) {
        activeStyle = {
          backgroundColor: '#ffffff',
          color: 'black',
          textDecoration: 'underline'
        };
      } else {
        activeStyle = {
          backgroundColor: 'transparent',
          color: '#ee162d',
          border: 'none',
          textDecoration: 'none'
        };
      }
      if (typeof page === 'string') {
        activeStyle = {
          backgroundColor: 'white',
          color: '#ee162d',
          border: 'none'
        };
      }
      return (
        <li className='page-item-4-4-31'>
          <a href='#' onClick={handleClick} style={activeStyle}>
            {page}
          </a>
        </li>
      );
    };

    const options = {
      pageButtonRenderer,
      hideSizePerPage: this.state.data.length <= 5,
      hidePageListOnlyOnePage: true,
      sizePerPageList: [
        {
          text: '5',
          value: 5
        },
        {
          text: '10',
          value: 10
        },
        {
          text: '50',
          value: 50
        }
      ]
    };

    if (!this.state.columns) {
      return (
        <div className='sem-resultado'>Sem resultados para apresentar</div>
      );
    }

    const {
      showSelectRadio,
      hasExpandRow,
      getExpandedRowValue,
      selected,
      nonExpandableIds,
    } = this.props;

    const toggleExpandRow = row => {
      row.reutilizarDossier = row.hasOwnProperty('reutilizarDossier')
        ? !row.reutilizarDossier
        : true;
      getExpandedRowValue(row);
    };

    const selectRow = {
      mode: 'radio',
      classes: 'selecionada',
      clickToExpand: false,
      clickToSelect: true,
      selected: this.state.selected,
      onSelect: this.handleOnSelect,
      selectionRenderer: ({ mode, ...rest }) => (
        <label
          className='input__contaier'
          style={{
            display: showSelectRadio ? 'block' : 'none',
          }}
        >
          <input type={mode} {...rest} />
          <span className='checkmark' />
        </label>
      )
    };


    const expandRow = {
      renderer: row => (
        <label>
          <input
            type='checkbox'
            value={
              row.hasOwnProperty('reutilizarDossier')
                ? !row.reutilizarDossier
                : true
            }
            onChange={() => toggleExpandRow(row)}
          />
          <p>
          Reutilizar processo completo de encaminhamento pressupõe utilizar documentos, histórico e acessos das pessoas já envolvidas.
          </p>
        </label>
      ),
      expanded: this.state.selected.filter(gridId => {
        if(nonExpandableIds){
          const notExpandable = nonExpandableIds.find(id=>gridId===id);
          if(notExpandable){
            return false;
          }
        }
        return (gridId === this.state.selected[0]);
      }),
      nonExpandable: this.state.selected.filter(gridId => {
        return gridId !== this.state.selected[0];
      }),
      showExpandColumn: true,
      expandByColumnOnly: true,
      expandColumnPosition: 'right'
    };

    const resolveExpandRow = (customExpandRow) => ({ ...expandRow, ...customExpandRow });

    const removeDuplicity = (data) => data.reduce((array,newItem)=>{
        const find = array.find(item=>newItem.Id===item.Id);
        return find? array: [...array,newItem];
    },[]);
    
    const rowEvents = {
      onMouseEnter: (event, row, rowIndex) => {
        const rowEl = document.querySelectorAll(`.row-class-${stringCleaner(row.rowInfos.id)}`)
        if (rowEl) {
          const last = rowEl.length === 0 ? 0 : rowEl.length - 1
          rowEl[last].classList.add('hovering');
        }
        if (this.props.contextMenuItems) {
          this.addDotsToRow(row.rowInfos.id)
        }
      },
      onMouseLeave: (event, row, rowIndex) => {
        const rowEl = document.querySelectorAll(`.row-class-${stringCleaner(row.rowInfos.id)}`)
        if (rowEl) {
          const last = rowEl.length === 0 ? 0 : rowEl.length - 1
          rowEl[last].classList.remove('hovering');
        }

        if (this.props.contextMenuItems) {
          this.removeDotsToRow(row.rowInfos.id)
        }

      },
      onContextMenu: (event, row) => {
        event.preventDefault();
        if (this.props.contextMenuItems) {
          this.onContextMenu(event, row)
        }
      }
    }

    const tabbleOptions = {
      keyField: 'Id',
      data: removeDuplicity(this.state.data),
      columns: this.state.columns,
      bordered: false,
      loading: true,
      selectRow: selectRow,
      rowEvents,
      expandRow: hasExpandRow ? resolveExpandRow(this.props.customExpandRow) : undefined,
      onTableChange: this.handleTableChange,
      pagination: paginationFactory(options),
      rowClasses: (row, rowIndex) => `row-class-${stringCleaner(row.rowInfos.id)}`,
      rowStyle: {
        backgroundColor:
          selected && this.state.data.length === 1 ? '#f7f7f7' : '#fff'
      }
    };

    return (
      <div className={`ReactEdpComponents__table-container-v4-6-06 ${this.props.subTable} ${this.props.exclusiveClass}`}>
        <BootstrapTable {...tabbleOptions} />
      </div>
    );
  }
}
