import React from "react";
import { TextField } from "@material-ui/core";
import Icon from "react-edp-icons";

import "./search-field.css";
import "./search-field.scss";

//TEST data: TESTE-DDOC086@edp.pt
export class SearchField extends React.Component {
	constructor(props) {
		super(props);

		this.state = {
			value: "",
			showSearchResults: false,
		};

		this.sessionStorage = sessionStorage.getItem("name") && JSON.parse(sessionStorage.getItem("name"));
		console.log(this.sessionStorage);
	}

	componentDidMount() {
		document.addEventListener("mousedown", this.handleClickOutside);
	}

	componentWillUnmount() {
		document.removeEventListener("mousedown", this.handleClickOutside);
	}

	onChange = e => {
		this.setState({ value: e.target.value });
	};

	onClick = e => {
		e.preventDefault();
		this.searchUsers();
	};

	setWrapperRef = node => {
		this.wrapperRef = node;
	};

	handleClickOutside = event => {
		if (this.wrapperRef && !this.wrapperRef.contains(event.target)) {
			this.setState({ showSearchResults: false });
		}
	};

	//TODO: check sessionStorage static substitute
	searchUsers = () => {
		if (this.sessionStorage) {
			const companyId = this.sessionStorage.company.id;
			this.props.seachUsersByNameOrId(this.state.value, this.props.environment, companyId);
		} else {
			this.props.seachUsersByNameOrId(this.state.value, this.props.environment, "EDP, SA");
		}
		this.setState({ showSearchResults: true });
	};

	onKeyUp = e => {
		e.preventDefault();
		const code = e.keyCode ? e.keyCode : e.which;
		if (code === 13) {
			this.searchUsers();
		}
	};

	render() {
		const { value, showSearchResults } = this.state;
		const { results, addUser, userSearchLoading } = this.props;
		return (
			<div>
				<div className="search-text__wrapper">
					<TextField
						label="Pesquise contactos internos por Nome"
						type="search"
						value={value}
						onChange={this.onChange}
						onKeyUp={this.onKeyUp}
						fullWidth
						margin="none"
						InputLabelProps={{
							shrink: value == null || value == "" ? false : true,
						}}
					/>
					<button type="button" onClick={this.onClick}>
						<Icon icon="PESQUISAR" color={"#EE162D"} size={40} />
					</button>
				</div>
				{showSearchResults && results && !userSearchLoading && (
					<div className="search-text__results" ref={node => this.setWrapperRef(node)}>
						{results.map((r, i) => (
							<h1 key={`${r.id}${r.poUser}`} className="search-text__result" onClick={() => addUser(r)}>
								{r.displayName}
							</h1>
						))}
					</div>
				)}
			</div>
		);
	}
}
