import React from 'react';
// import { connect } from 'react-redux';

import { setPropertiesToSave, setNextStep, notifySave, setGoNext } from '../../service-calls/stepActions';
import { setStepDescription } from '../../service-calls/stepHeader';
import { RESPONSES } from './ResponsesConstants';
import SelecionarUtilizador from '../SelecionarUtilizador';
import { callModalMessages } from './DragComponent/constants/util';

import "./UserSelectionForward.scss";
// import "./UserSelectionForward.css";

import RadioItemForward from './RadioItem/RadioItemForward';
import Checkbox from "@material-ui/core/Checkbox";
import { Button, Tabs, Tab } from 'react-bootstrap';
import DatePicker from 'react-datepicker';
import Icon from 'react-edp-icons';

import R from 'ramda';

import getUsersPO from './getUsersPO'


export const CONTRATOS_DRAG_CC = "contatosDragCC";
export const CONTRATOS_DRAG_UTILIZADORES = "contatosDragUtilizadores";
export class UserSelectionForward extends React.Component {
    constructor(props) {
        super(props);

        this.props.setStepDescription(
            'Para o encaminhamento de um documento, é necessário que seja selecionado um ou mais destinatários.A seleção do destinatário poderá ser feita através da pesquisa do nome ou EDP User ID, ou da navegação na estrutura organizativa da empresa ou nas listas disponíveis.'
        );
        this.properties = {
            EDP_usersACL: [],
            EDP_usersCC:[],
            EDP_POUsersArray: [],
            EDP_UNUsersArray: [],
            EDP_UOUsersArray: [],
        };
		this.state = {
            key: 1,
            environment: "",
            EDP_inDocComments: false,
            radioButton: "user",
            labelColor: "#000",
            textAreaColor: "#ddd",
            EDP_eSecreto: this.props.icmWorkItem.attributes.EDP_eSecreto,
            EDP_Responses: this.props.icmWorkItem.attributes.EDP_Responses,
            icmWorkItem: this.props.icmWorkItem,
            EDP_MensagemdeNotificacao: !R.isEmpty(this.props.icmWorkItem)
                ? this.props.icmWorkItem.attributes.EDP_MensagemdeNotificacao
                : '',
            EDP_DataLimite: !R.isEmpty(this.props.icmWorkItem)
                ? this.props.icmWorkItem.attributes.EDP_DataLimite
                : new Date()
        };  
        
        this.options = {
            user: 'Pessoa(s)',
            org: 'Organização (Grupos/Equipas)'
        };

        const { EDP_eSecreto, EDP_Responses } = this.props.icmWorkItem.attributes;

        this.personalForward = EDP_eSecreto;

        if(
            EDP_eSecreto 
            || EDP_Responses === "SolicitarParecer" 
            || EDP_Responses === "SolicitarDecisão" 
            || EDP_Responses === "Outrosfins"){
            this.radioButtonDisable = true;
        } else {
            this.radioButtonDisable = false;
        }
    }

    componentWillMount() {
        ecm.model.Request
            .invokePluginService("EdpECMenv", "GetConfigurationServiceEDPGroups", {
            requestCompleteCallback: (response) => {
                const environment = response.configuration["0"].value.toLowerCase();
                this.setState({environment: environment});
            }
        });
    }

    handleSelect = (key) => {
        this.setState({ key: key });
    };

	handleCheckBox = () => {
        this.setState({EDP_inDocComments: !this.state.EDP_inDocComments});
    };

    handleModal = () => {
        console.log('==== ')
        const userSeleted = this.props.contatosDragUtilizadores.filter(({ selected }) => selected);
        if (userSeleted && userSeleted.length) {
            if (userSeleted[0].hasOwnProperty("usersComAcesso")) {
                // abre o modal se já existirem dados
                window.icnReactService.sendMessage({
                    message: 'render_FORWARD_MODAL',
                    usersList: this.properties.usersList,
                    EDP_eSecreto: this.state.EDP_eSecreto,
                    personalForward: this.personalForward,
                    radioButton: this.state.radioButton,
                    callBackSuccess: personalForward => this.personalForward = personalForward
                });
            } else {
                Promise
                    .all(userSeleted.map(user => getUsersPO(user)))
                    .then(users => {
                        this.properties = { ...this.properties, usersList: users };
                        window.icnReactService.sendMessage({
                            message: 'render_FORWARD_MODAL',
                            usersList: this.properties.usersList,
                            EDP_eSecreto: this.state.EDP_eSecreto,
                            personalForward: this.personalForward,
                            radioButton: this.state.radioButton,
                            callBackSuccess: personalForward => this.personalForward = personalForward
                        });
                    });
            }
        }
    };

	componentDidUpdate(prevProps) {
        const { notifySave } = this.props; 
        if (notifySave) {
            this.submitForm();
        }

        this.properties = { ...this.getPropsToSave(this.props.contatosDragUtilizadores, this.props.contatosDragCC) };

        if (!R.equals(this.props.icmWorkItem, prevProps.icmWorkItem)) {
            if (!R.isEmpty(this.props.icmWorkItem)) {
                this.setState({
                    EDP_MensagemdeNotificacao: this.props.icmWorkItem.attributes
                        .EDP_MensagemdeNotificacao,
                    EDP_DataLimite: this.props.icmWorkItem.attributes
                        .EDP_DataLimite
                        ? this.props.icmWorkItem.attributes.EDP_DataLimite
                        : new Date()
                });
            }
        }
    }

    getPropsToSave(userSeleted, userSelectedCC) {
        const EDP_usersACL = [];
        const EDP_usersCC = [];
        const EDP_POUsersArray = [];
        const EDP_UNUsersArray = [];
        const EDP_UOUsersArray = [];
        const selectedUser = userSeleted.filter(({ selected }) => selected);
        const selectedUserCC = userSelectedCC.filter(({ selected }) => selected);
        const userFullProps = selectedUser;

        selectedUser.forEach(user => {
            EDP_usersACL.push(user.shortName);
            EDP_POUsersArray.push(user.PO_ShortName);
            EDP_UNUsersArray.push(user.UN_ShortName);
            EDP_UOUsersArray.push(user.UO_ShortName);
        });

        selectedUserCC.forEach(user => {
            EDP_usersCC.push(user.shortName);

        });

        const properties = {
            ...{
                EDP_usersCC,
                EDP_usersACL,
                EDP_POUsersArray,
                EDP_UNUsersArray,
                EDP_UOUsersArray,
            },
            usersList: [...userFullProps]
        }
        console.log('==== propretiesToSave USER_SELECTION_FORWARD ======', properties)

        if(this.personalForward && properties.usersList.length > 1) {
            this.personalForward = false
        }

        return properties
    }
		
    submitForm() {
        const { usersList } = this.properties || [];
        const { EDP_MensagemdeNotificacao, EDP_Responses } = this.state;

        if((EDP_MensagemdeNotificacao) || EDP_Responses === "Daracessosemnotificação"){
            if(usersList.length){
                const {
                    radioButton,
                    EDP_Responses,
                    EDP_MensagemdeNotificacao, 
                    EDP_inDocComments,
                    EDP_DataLimite           
                } = this.state;
        
                // carrega apenas o UsersACL
                if(this.personalForward){
                    const properties = {
                        EDP_Responses,
                        EDP_MensagemdeNotificacao,
                        EDP_inDocComments,
                        EDP_DataLimite: EDP_DataLimite
                            ? EDP_DataLimite
                            : this.props.icmWorkItem.attributes.EDP_DataLimite,
                        EDP_nrUtilizadores: this.properties.EDP_POUsersArray
                            .length,
                        EDP_usersACL: this.properties.EDP_usersACL,
                        EDP_usersCC: this.properties.EDP_usersCC
                    };
            
                    console.log( '==== SUBMIT (only UsersACL) ===',
                        properties
                    );
                    this.props.setPropertiesToSave(properties);
                }else{
                    if(radioButton === "org"){
                    // carrega apenas o POArray
                    const properties = {
                        EDP_Responses,
                        EDP_MensagemdeNotificacao,
                        EDP_inDocComments,
                        EDP_DataLimite: EDP_DataLimite
                            ? EDP_DataLimite
                            : this.props.icmWorkItem.attributes.EDP_DataLimite,
                        EDP_nrUtilizadores: this.properties.EDP_POUsersArray
                            .length,
                        EDP_POUsersArray: this.properties.EDP_POUsersArray,
                    };
                    console.log('==== SUBMIT (Only PO) ===',
                        properties
                    );
                    this.props.setPropertiesToSave(properties);
                    }
                    // carrega UsersACL e o POArray
                    if(radioButton === "user"){
                        const properties = {
                            EDP_Responses,
                            EDP_MensagemdeNotificacao,
                            EDP_inDocComments,
                            EDP_DataLimite: EDP_DataLimite
                                ? EDP_DataLimite
                                : this.props.icmWorkItem.attributes.EDP_DataLimite,
                            EDP_nrUtilizadores: this.properties.EDP_POUsersArray
                                .length,
                            EDP_POUsersArray: this.properties.EDP_POUsersArray,
                            EDP_usersACL: this.properties.EDP_usersACL,
                            EDP_usersCC: this.properties.EDP_usersCC
                        };
                        console.log('==== SUBMIT (UsersACL and PO) ===',
                            properties
                        );
                        this.props.setPropertiesToSave(properties);
                    }
                }
            }else{
                callModalMessages({
                    messageType: 'alert',
                    messageBody: 'Atenção: De modo a avançar com o processo, selecione o(s) utilizador(es) de destino.'
                });
            }
        }else{
            if (!this.props.onlySave) {
                document.getElementById("error-forward-text-area").style.display = "block";
                this.props.setNotifySave(false);
                this.props.setGoNext(false);
            } else {
                const { 
                    EDP_Responses,
                    EDP_MensagemdeNotificacao, 
                    EDP_inDocComments,
                    EDP_DataLimite           
                } = this.state;
                const properties = {
                    EDP_Responses,
                    EDP_MensagemdeNotificacao,
                    EDP_inDocComments,
                    EDP_DataLimite: EDP_DataLimite
                        ? EDP_DataLimite
                        : this.props.icmWorkItem.attributes.EDP_DataLimite
                };
                this.props.setPropertiesToSave(properties);
            }
        }                      
    }

    onPublicacaoSubmit = userToSave => {
        console.log('==== ON PUBLICACAO SUBMIT Encaminhar ===', userToSave);
        this.properties = { ...userToSave };
    };

    setMessage = value => {
        document.getElementById("error-forward-text-area").style.display = "none";
        this.setState({labelColor: "#000"});
        this.setState({textAreaColor: "#ddd"});
        this.setState({ EDP_MensagemdeNotificacao: value });
    };
    setDate = value => {
        this.setState({ EDP_DataLimite: value });
    };
    handleChange = e => {
        const { value } = e.target || '';   
        this.setState({ radioButton: value});   

        if(value === "org") {
            this.personalForward = false;
        }
    };

    render() {
        const {  
            key,
            EDP_Responses,
            EDP_DataLimite,
            EDP_eSecreto
        } = this.state;

        const {  EDP_MensagemdeNotificacao } = this.state || '';

        const userSeleted = this.props.contatosDragUtilizadores.filter(({ selected }) => selected);

        const now = new Date();
        const datePickerOptions = {
            minDate: now,
            showTimeSelect: true,
            timeFormat: 'HH:mm',
            timeIntervals: 15,
            dateFormat: 'dd/MM/yyyy HH:mm',
            timeCaption: 'Hora'
        };

        const { textAreaColor } = this.state || "#ddd";

        // condição para mostrar o campo da Data Limite
        let show_EDP_DataLimite = (EDP_Responses === "InformarLeitura" ||
                                    EDP_Responses === "SolicitarParecer" ||
                                    EDP_Responses === "SolicitarDecisão" ||
                                    EDP_Responses === "OutrosFins" 
                                ) ? true : false;
        
        let tabsEncaminhar = [
            {
                key: 1,
                title: RESPONSES[EDP_Responses],
                content: (
                    <div>
                        <div className="radioButton-parent-div">
                            <span className="radioButton-child-p">Como pretende pesquisar?</span>
                            {Object.entries(this.options).map(([key, val]) => (
                                <div  className="radioButton-child-div">
                                    <RadioItemForward
                                    radioButtonDisable={this.radioButtonDisable}
                                    key={key}
                                    value={key}
                                    checked={
                                        this.state.radioButton === key
                                    }
                                    handleChange={this.handleChange}
                                    label={val}
                                />
                                </div> 
                            ))}
                        </div>
                        <SelecionarUtilizador conteudoUtilizadores={CONTRATOS_DRAG_UTILIZADORES} hideGroupRow={true} key="1" onSubmit={this.onPublicacaoSubmit} 
                            encaminhar={true} eSecreto={EDP_eSecreto} dontSave />

                        <div className="text-right">
                            <Button onClick={this.handleModal} className="btn-options" id="button-forward-options" disabled={!userSeleted.length}>
                                Opções de Encaminhamento
                            </Button>
                        </div>

                        <div className="col-md-12 msg">
                            {EDP_Responses != "Daracessosemnotificação" ? 
                                (<div>
                                    <div className="text-area__container">
                                        <h2 className="formatAddMsg">Adicionar Mensagem</h2>
                                        <textarea
                                            rows="3"
                                            name="Adicionar Mensagem"
                                            value={EDP_MensagemdeNotificacao}
                                            style={{border: "1px solid" + textAreaColor}}
                                            onChange={event =>
                                                this.setMessage(event.target.value)
                                            }
                                        />
                                        <p id="error-forward-text-area" style={{color: "#f00",display:"none"}}>Campo obrigatório</p>
                                    </div>   
                                    <div style={{margin: "-40px 0 25px -15px"}}>
                                        <Checkbox color="default" onClick={this.handleCheckBox} />
                                        <span style={{color: "#CCCCCC", margin: "14px 0 0 -10px"}}>
                                            Selecione, caso pretenda converter esta mensagem em documento (atribuir carácter documental)
                                        </span> 
                                    </div> 
                                </div>) : null
                            }
                                    
                            {show_EDP_DataLimite && (
                                <div className="data-area__container">
                                    <label>Data/hora limite de resposta</label>
                                    <div className="date-picker__container">
                                        <div className="__start">
                                            <DatePicker
                                                {...datePickerOptions}
                                                selected={EDP_DataLimite}
                                                onChange={value => this.setDate(value)}
                                            />
                                            <div className="icon__container">
                                                <Icon
                                                    icon="DATA_HORA"
                                                    size={40}
                                                    color="#ccc"
                                                />
                                            </div>
                                        </div>
                                    </div>
                                </div>
                            )}
                            
                        </div>
                    </div>
                )
            },
            {
                key: 2,
                title: "C/Conhecimento",
                content: (
                    <div style={{marginTop: "10px"}}>
                        <SelecionarUtilizador  conteudoUtilizadores={CONTRATOS_DRAG_CC} key="2" onSubmit={this.onPublicacaoSubmit}
                            tabConhecimento={true} encaminhar={true} dontSave />
                    </div>
                    
                )
            }
        ];

        return ( 
            <Tabs
                activeKey={key}
                onSelect={this.handleSelect}
                id="step-header__tab"
            > 
            {tabsEncaminhar.map(tabsEncaminhar => (
                <Tab
                    key={tabsEncaminhar.key}
                    eventKey={tabsEncaminhar.key}
                    title={tabsEncaminhar.title}
                    className="tabs-encaminhar"
                >
                    {tabsEncaminhar.content}
                </Tab>
            ))}
            </Tabs>            
        );
    }
}

const mapStateToProps = state => {
    return {
        icmWorkItem: state.stepReducer.icmWorkItem,
        notifySave: state.stepReducer.notifySave,
        onlySave: state.stepReducer.onlySave,
        contatosDragUtilizadores: state.stepDelegationReducer.contatosDragUtilizadores,
        contatosDragCC: state.stepDelegationReducer.contatosDragCC
    };
};
const mapDispatchToProps = dispatch => {
    return {
        setNotifySave: value => dispatch(notifySave(value)),
        setPropertiesToSave: properties =>
            dispatch(setPropertiesToSave(properties)),
        setNextStep: step => dispatch(setNextStep(step)),
        setStepDescription: description =>
            dispatch(setStepDescription(description)),
        setGoNext: value => dispatch(setGoNext(value)),
    };
};

export default connect(
    mapStateToProps,
    mapDispatchToProps
)(UserSelectionForward);
