import { redirectToFeature } from 'react-edp-components';
import { promisseAmbiente } from './navigator';
import {
    STEP_ASSOCIAR_DOCUMENTOS,
    STEP_DIVULGAR_DOCUMENTOS,
    STEP_ENCAMINHAR_DOCUMENTOS
} from '../app';
import { promiseRetrieveItem, isDelegation } from './addDocument';
export const SET_WORK_ITEM = 'SET_WORK_ITEM';
export const SET_NEXT_STEP = 'SET_NEXT_STEP';
export const ADD_STEP = 'ADD_STEP';
export const SET_ACTUAL_STEP = 'SET_ACTUAL_STEP';
export const SET_SAVED_PROPERTIES = 'SET_SAVED_PROPERTIES';
export const LOAD_ACTUAL_STEP = 'LOAD_ACTUAL_STEP';
export const SET_PROPERTIES_TO_SAVE = 'SET_PROPERTIES_TO_SAVE';
export const SET_NOTIFY_SAVE = 'SET_NOTIFY_SAVE';
export const SET_GO_NEXT = 'SET_GO_NEXT';
export const SET_GO_BACK = 'SET_GO_BACK';
export const SET_NOTIFY_BACK = 'SET_NOTIFY_BACK';
export const SET_BEFORE_BACK = 'SET_BEFORE_BACK';
export const RESET_APP_STATE = 'RESET_APP_STATE';
export const SET_ONLY_SAVE = 'SET_ONLY_SAVE';


const _promiseRetrieveStep = icmWorkItem =>
    new Promise(resolve => {
        icmWorkItem.retrieveStep(icmWorkItem => resolve(icmWorkItem));
    });

const _promiseLockStep = icmWorkItem =>
    new Promise(resolve => {
        const userProperties = JSON.parse(sessionStorage.name);
        if (isDelegation(userProperties, 'onBehalfOf')) {
            console.log('IS DELEGATION, locking by force');
            icmWorkItem.overrideLockStep(icmWorkItem => resolve(icmWorkItem));
        } else {
            icmWorkItem.lockStep(icmWorkItem => resolve(icmWorkItem));
        }
    });

export const loadStep = stepLoaded => ({
    type: LOAD_ACTUAL_STEP,
    stepLoaded
});

export const setActualStep = actualStep => ({
    type: SET_ACTUAL_STEP,
    actualStep
});

export const setSavedProperties = savedProperties => ({
    type: SET_SAVED_PROPERTIES,
    savedProperties
});

export const setGoNext = goNext => ({
    type: SET_GO_NEXT,
    goNext
});

export const setGoBack = goBack => ({
    type: SET_GO_BACK,
    goBack
});


export const setNotifyBack = notifyBack => ({
    type: SET_NOTIFY_BACK,
    notifyBack
});

export const setBeforeBack = beforeBack => ({
    type: SET_BEFORE_BACK,
    beforeBack
});


export const setOnlySave = onlySave => ({
    type: SET_ONLY_SAVE,
    onlySave
});


const _setSessionValues = propertiesCollection => {
    const { organizationUnit, organizationPosition, company } = JSON.parse(
        sessionStorage.name
    );

    if (propertiesCollection.EDP_UnidadeOrganizativa != undefined)
        propertiesCollection.EDP_UnidadeOrganizativa.setValue(
            organizationUnit.id
        );
    if (propertiesCollection.EDP_PosicaoOrganizativa != undefined)
        propertiesCollection.EDP_PosicaoOrganizativa.setValue(
            organizationPosition.id
        );
    if (propertiesCollection.EDP_Empresa != undefined)
        propertiesCollection.EDP_Empresa.setValue(company.id);
};

export const saveStep = () => (dispatch, getState) => {
    const {
        icmWorkItem,
        propertiesToSave,
        actualStep,
        goNext
    } = getState().stepReducer;
    const editable = icmWorkItem.createEditable();
    propertiesToSave.EDP_reactWizardSteps = actualStep;

    console.log('============PROPERTIES TO SAVE =======', propertiesToSave);

    Object.keys(propertiesToSave).forEach(propertie => {
        editable.propertiesCollection[propertie].setValue(
            propertiesToSave[propertie]
        );
    });

    _setSessionValues(editable.propertiesCollection);

    console.log('=====BEFORE SAVE====', editable.propertiesCollection);

    editable.saveStep(a => {
        if (goNext) {
            dispatch(goNextStep());
            dispatch(setGoNext(false));
        }
    });
    if (propertiesToSave.EDP_Responses) {
        dispatch(setSavedProperties(propertiesToSave));
    }
    dispatch(setPropertiesToSave({}));
    dispatch(notifySave(false));
    dispatch(setOnlySave(false));
};

export const setNextStep = nextStep => ({
    type: SET_NEXT_STEP,
    nextStep
});

export const goNextStep = () => (dispatch, getState) => {
    if (getState().stepReducer.nextStep) {
        dispatch(addStep(getState().stepReducer.nextStep));
        dispatch(setNextStep(''));
    } else {
        dispatch(completeStep(selectedResponses.SUBMETER));
    }
};

export const notifySave = value => ({
    type: SET_NOTIFY_SAVE,
    value: value
});

const _setWorkItem = icmWorkItem => ({
    type: SET_WORK_ITEM,
    icmWorkItem
});
export const setPropertiesToSave = propertiesToSave => ({
    type: SET_PROPERTIES_TO_SAVE,
    propertiesToSave
});

export const addStep = newStep => ({
    type: ADD_STEP,
    newStep
});

export const resetState = () => ({
    type: RESET_APP_STATE
});

const _promiseValidatePersonalData = icmWorkItem =>
    new Promise(resolve => {
        const serviceParams = {};
        serviceParams.taskStep = 'open';
        serviceParams.caseFolderID = icmWorkItem.caseFolderId;
        serviceParams.repositoryID = ecm.model.desktop.getDefaultRepositoryId();
        serviceParams.workflowName = 'na';
        serviceParams.vsID = 'na';
        serviceParams.nrUsersSeleccionados = 'na';
        serviceParams.taskResponse = 'na';
        serviceParams.envPrefix = 'na';
        serviceParams.realmName = 'na';
        serviceParams.lockedUser = 'na';
        serviceParams.Empresa = 'na';
        serviceParams.UO = 'na';
        serviceParams.PO = 'na';
        console.log('======VALIDATING PERSONAL DATA=========', serviceParams);
        ecm.model.Request.invokePluginService(
            'EdpECMServices',
            'TaskActionValidateService',
            {
                requestParams: serviceParams,

                requestCompleteCallback: responseTaskActionValidateService => {
                    console.log(
                        '====RESPONSE VALIDATE PERSONAL DATA====',
                        responseTaskActionValidateService
                    );
                    resolve(responseTaskActionValidateService);
                }
            }
        );
    });

export const promiseGetCurrentItemById = itemId =>
    new Promise((resolve, reject) => {
        console.log('====ASKING FOR ITEM  ', itemId);
        const repositoryId = ecm.model.desktop.getDefaultRepositoryId();
        getCurrentItemByID(repositoryId, itemId, item => {
            console.log('===resolvedVERSIONITEM=== ', item);
            resolve(item);
        });
    });
export const startSetWorkItem = icmWorkItem => (dispatch, getState) => {
    _promiseRetrieveStep(icmWorkItem)
        .then(_promiseLockStep)
        .then(icmWorkItem => {
            _promiseValidatePersonalData(icmWorkItem).then(
                responseTaskActionValidateService => {
                    if (responseTaskActionValidateService.length !== 0) {
                        window.icnReactService.sendMessage({
                            message: 'render_modalMessagesReact',

                            jsonArray: responseTaskActionValidateService
                        });
                    }
                }
            );
            dispatch(_setWorkItem(icmWorkItem));
            dispatch(loadStep(icmWorkItem.attributes.EDP_reactWizardSteps));
        });
};
export const selectedResponses = { SUBMETER: 'Submeter', CANCELAR: 'Cancelar' };
// Publicardocumento, Enviarparapublicacao, Enviarpararevisao, Devolver, Documentorevist, Encaminhamento
export const POSSIBLE_EDP_RESPONSES = {
    PUBLICAR_DOCUMENTO: 'Publicardocumento',
    ENVIAR_PUBLICACAO: 'Enviarparapublicacao',
    ENVIAR_REVISAO: 'Enviarpararevisao',
    DEVOLVER: 'Devolver',
    REVISTO: 'Documentorevisto',
    ALTERAR_PROPRIEDADES: 'AlterarPropriedades',
    REJEITAR: 'Rejeitar',
    SOLICITAR_PARECER: 'SolicitarParecer',
    SOLICITAR_DECISAO: 'SolicitarDecisão',
    DAR_ACESSO_SEM_NOTIFICACAO: 'Daracessosemnotificação',
    ENCAMINHAMENTO_GENERICO: /Informar|Informarcompedidodeleitura|Outrosfins/,
    DIVULGAR_CARTA: 'DivulgarCarta',
    DIVULGAR_FAX: 'DivulgarFax',
    DIVULGAR_EMAIL: 'DivulgarEmail'
};
const _showModal = icmWorkItem => {
    let mensagem = '';
    let itemsNextStep = '';
    const { caseFolderId, attributes, stepName, ecmWorkItem } = icmWorkItem;
    const { workclass_name } = ecmWorkItem;
    console.log('====STEP NAME====', stepName);
    const {
        EDP_DocumentoRepublicado,
        EDP_Responses,
        EDP_Assunto,
        EDP_IdDocPrincipal,
        EDP_isPowerUser,
        EDP_isDonoDocumento,
        EDP_vsIdInitiatingDoc,
        EDP_Processo
    } = attributes;
    const assunto = EDP_Assunto ? EDP_Assunto : '';
    const publicacao = EDP_DocumentoRepublicado ? 'Republicado' : 'Publicado';
    console.log('REPONSE', EDP_Responses);
    console.log('======= EDP_Processo =======', EDP_Processo);
    if (EDP_Processo === 'Encaminhamento') {
        mensagem = definirMensagemEncaminhamento(
            assunto,
            workclass_name,
            EDP_Responses,
            stepName
        );
        itemsNextStep = nextSteps(
            stepName,
            EDP_IdDocPrincipal,
            caseFolderId,
            EDP_vsIdInitiatingDoc,
            EDP_Responses
        );
    }
    if  (_isAssociarDocumentos(icmWorkItem)) {
        console.log('ASSOCIAR DOCUMENTOS');
        window.icnReactService.sendMessage({
            message: 'render_NEXTSTEP_MODAL',
            icon: 'MSG_SUCESSO',
            caseFolderID: caseFolderId,
            titleModal: 'Documentos(s) associados(s) com sucesso.',
            callBackCloseStep: () => {
                console.log('CALLBACK CALLED');
                redirectToFeature(ecm.model.desktop.defaultFeature);
                window.icnReactService.sendMessage({
                    message: 'remove_eDP_ReactStepProcessors'
                });
            },

            itemsNextStep: nextSteps(
                stepName,
                EDP_IdDocPrincipal,
                caseFolderId,
                EDP_vsIdInitiatingDoc,
                EDP_Responses
            )
        });
    }
    else {
        switch (EDP_Responses) {
            case POSSIBLE_EDP_RESPONSES.PUBLICAR_DOCUMENTO :
                window.icnReactService.sendMessage({
                    message: 'render_NEXTSTEP_MODAL',
                    icon: 'MSG_SUCESSO',
                    caseFolderID: caseFolderId,
                    titleModal: publicacao + ' com sucesso ' + assunto,
                    callBackCloseStep: () => {
                        console.log('CALLBACK CALLED');
                        redirectToFeature(ecm.model.desktop.defaultFeature);
                        window.icnReactService.sendMessage({
                            message: 'remove_eDP_ReactStepProcessors'
                        });
                    },
    
                    itemsNextStep: nextSteps(
                        stepName,
                        EDP_IdDocPrincipal,
                        caseFolderId,
                        EDP_vsIdInitiatingDoc,
                        EDP_Responses
                    )
                });
                break;
            case POSSIBLE_EDP_RESPONSES.ENVIAR_PUBLICACAO:
                mensagem = [
                    { title: true, message: `Enviado com sucesso` },
                    `Publicação: Pedido de Publicação `,
                    assunto
                ];
                break;
            case POSSIBLE_EDP_RESPONSES.ENVIAR_REVISAO:
                mensagem = [
                    { title: true, message: `Enviado com sucesso` },
                    `Publicação: Pedido de Revisão 
                      `,
                    assunto
                ];
                break;
            case POSSIBLE_EDP_RESPONSES.REVISTO:
                mensagem = [
                    { title: true, message: `Enviado com sucesso` },
                    `Publicação: Documento Revisto 
                      `,
                    assunto
                ];
                break;
            case POSSIBLE_EDP_RESPONSES.DEVOLVER:
                mensagem = [
                    { title: true, message: `Enviado com sucesso` },
                    `Publicação: Documento Devolvido 
                      `,
                    assunto
                ];
                break;
            case POSSIBLE_EDP_RESPONSES.REJEITAR:
                mensagem = [
                    { title: true, message: `Enviado com sucesso` },
                    `Encaminhamento : Pedido de Parecer - Devolução de
                      ${EDP_Responses} - ${assunto}`
                ];
                break;
            case POSSIBLE_EDP_RESPONSES.ALTERAR_PROPRIEDADES:
                mensagem = definirMensagemAlterarPropriedades(
                    assunto,
                    workclass_name,
                    EDP_isPowerUser,
                    EDP_isDonoDocumento
                );
    
                break;
            case POSSIBLE_EDP_RESPONSES.SOLICITAR_PARECER:
                mensagem = [
                    `Encaminhado com sucesso`,
                    `Encaminhamento: Solicitação de Parecer
                    ${assunto}`
                ];
                itemsNextStep = nextSteps(
                    stepName,
                    EDP_IdDocPrincipal,
                    caseFolderId,
                    EDP_vsIdInitiatingDoc,
                    EDP_Responses
                );
                break;
            case POSSIBLE_EDP_RESPONSES.SOLICITAR_DECISAO:
                mensagem = [
                    `Encaminhado com sucesso`,
                    `Encaminhamento: Solicitação de Decisão
                    ${assunto}`
                ];
                itemsNextStep = nextSteps(
                    stepName,
                    EDP_IdDocPrincipal,
                    caseFolderId,
                    EDP_vsIdInitiatingDoc,
                    EDP_Responses
                );
                break;
            case POSSIBLE_EDP_RESPONSES.DAR_ACESSO_SEM_NOTIFICACAO:
                mensagem = [
                    `Encaminhado com sucesso`,
                    `Encaminhamento: Dar acesso sem notificação
                    ${assunto}`
                ];
                itemsNextStep = nextSteps(
                    stepName,
                    EDP_IdDocPrincipal,
                    caseFolderId,
                    EDP_vsIdInitiatingDoc,
                    EDP_Responses
                );
                break;
            case POSSIBLE_EDP_RESPONSES.DIVULGAR_CARTA:
                mensagem = [`Publicado e Divulgado com sucesso`, `${assunto}`];
                itemsNextStep = nextSteps(
                    stepName,
                    EDP_IdDocPrincipal,
                    caseFolderId,
                    EDP_vsIdInitiatingDoc,
                    EDP_Responses
                );
                break;
            case POSSIBLE_EDP_RESPONSES.DIVULGAR_FAX:
                mensagem = [`Publicado e Divulgado com sucesso`, `${assunto}`];
                itemsNextStep = nextSteps(
                    stepName,
                    EDP_IdDocPrincipal,
                    caseFolderId,
                    EDP_vsIdInitiatingDoc,
                    EDP_Responses
                );
                break;
            case POSSIBLE_EDP_RESPONSES.DIVULGAR_EMAIL:
                mensagem = [`Publicado e Divulgado com sucesso`, `${assunto}`];
                itemsNextStep = nextSteps(
                    stepName,
                    EDP_IdDocPrincipal,
                    caseFolderId,
                    EDP_vsIdInitiatingDoc,
                    EDP_Responses
                );
                break;
        }
    }
   
    if (POSSIBLE_EDP_RESPONSES.PUBLICAR_DOCUMENTO !== EDP_Responses && !_isAssociarDocumentos(icmWorkItem)) {
        console.log('======RENDER MODAL MESSAGE REACT WITH =====', mensagem);
        console.log(
            '======RENDER MODAL MESSAGE REACT WITH itemsNextStep =====',
            itemsNextStep
        );
        window.icnReactService.sendMessage({
            message: 'render_modalMessagesReact',
            jsonArray: [
                {
                    messageType: 'success',
                    messageBody: mensagem,
                    itemsNextStep
                }
            ],
            callBackSuccess: () => {
                console.log('CALLBACK CALLED');
                redirectToFeature(ecm.model.desktop.defaultFeature);
                window.icnReactService.sendMessage({
                    message: 'remove_eDP_ReactStepProcessors'
                });
            },
            callBackCloseStep: () => {
                console.log('CALLBACK CALLED');
                redirectToFeature(ecm.model.desktop.defaultFeature);
                window.icnReactService.sendMessage({
                    message: 'remove_eDP_ReactStepProcessors'
                });
            }
        });
    }
};

const _abortComplete = responseTaskActionValidateService =>
    responseTaskActionValidateService.find(
        msge => msge.messageType === 'abort'
    );
const _promiseWFTaskValidation = icmWorkItem =>
    new Promise((resolve, reject) => {
        promisseAmbiente().then(ambiente => {
        const serviceParams = _resolveParams(icmWorkItem, ambiente);
        ecm.model.Request.invokePluginService('EdpECMServices', 'TaskActionValidateService', {
                requestParams: serviceParams,
                requestCompleteCallback: responseTaskActionValidateService => {
                    console.log(
                        '====RESPONSE VALIDATE WF DATA====',
                        responseTaskActionValidateService
                    );
                    if (responseTaskActionValidateService.length !== 0) {
                        if (_abortComplete(responseTaskActionValidateService)) {
                            window.icnReactService.sendMessage({
                                message: 'render_modalMessagesReact',
                                jsonArray: responseTaskActionValidateService
                            });
                            reject();
                        } else {
                            window.icnReactService.sendMessage({
                                message: 'render_modalMessagesReact',
                                jsonArray: responseTaskActionValidateService,
                                callBackSuccess: () => resolve(icmWorkItem)
                            });
                        }
                    } else {
                        resolve(icmWorkItem);
                    }
                }
            });
        })
    });
const _onBehalfOf = icmWorkItem =>
    new Promise(resolve => {
        console.log('====VERIFYING BEHALF_OF');
        const editable = icmWorkItem.createEditable();
        let userLogado = ecm.model.desktop.userId.toLowerCase();
        const userReturn = icmWorkItem.attributes.EDP_UserReturn.toLowerCase();
        if (userLogado !== userReturn) {
            console.log(`==== DELEGATE =====,${userLogado} to ${userReturn}`);
            editable.propertiesCollection.EDP_OnBehalf.setValue(userReturn);
            editable.saveStep(result => {
                const { icmWorkItem } = result;
                console.log('=======SAVED DELEGATION========', icmWorkItem);
                resolve(icmWorkItem.createEditable());
            });
        } else {
            resolve(editable);
        }
    });
export const completeStep = (response, callback) => (dispatch, getState) => {
    const { icmWorkItem, savedProperties } = getState().stepReducer;
    const editable = icmWorkItem.createEditable();

    switch (response) {
        case selectedResponses.CANCELAR:
            editable.selectedResponse = response;
            editable.completeStep(a => {
                console.log('===CANCEL  STEP=====', a);
                callback();
            });
            break;
        case selectedResponses.SUBMETER:
            submitWF(icmWorkItem, response, savedProperties, callback);
            break;
    }
    dispatch(setSavedProperties({}));
};

const textConfirmation = [
    {
        messageType: 'alert',
        messageBody: [
            'Atenção: solicitou partilhar um documento que contém dados pessoais. Deverá garantir que esses dados' +
                'apneas são transmitidos a quem deles necessite no âmbito da finalidade para a qual foram recolhidos e que o respetivo acesso e utilização' +
                'está conforme com o consentimento expresso pelos titulares dos dados em questão ou com base legal alternativa.' +
                ' Em caso de violação dos pressupostos acima indicados, a EDP poderá incorrer em sanções com impacto significativo para o negócio do Grupo.',
            'Confirma que pretende partilhar este documento ? '
        ]
    }
];
const submitWF = (icmWorkItem, response, savedProperties, callback) => {
    console.log('===TRYING TO COMPLET STEP with response=====', response);
    const { stepName } = icmWorkItem;
    if (
        stepName === STEP_ENCAMINHAR_DOCUMENTOS ||
        stepName === STEP_DIVULGAR_DOCUMENTOS
    ) {
        _promiseValidatePersonalData(icmWorkItem).then(
            responseTaskActionValidateService => {
                if (responseTaskActionValidateService.length > 0) {
                    window.icnReactService.sendMessage({
                        message: 'render_modalMessagesReact',
                        jsonArray: textConfirmation,
                        callBackSuccess: () => {
                            continueSubmitWF(icmWorkItem, response);
                            window.icnReactService.sendMessage({
                                message: 'remove_eDP_ReactStepProcessors'
                            });
                        },
                        callBackCloseStep: () => {
                            redirectToFeature(ecm.model.desktop.defaultFeature);
                            window.icnReactService.sendMessage({
                                message: 'remove_eDP_ReactStepProcessors'
                            });
                        },
                        textOK: 'Sim',
                        textCancel: 'Não'
                    });
                } else {
                    continueSubmitWF(icmWorkItem, response);
                }
            }
        );
    } else {
        continueSubmitWF(icmWorkItem, response);
    }
};

const _isAssociarDocumentos = icmWorkItem => {
    const { stepName } = icmWorkItem;
    return stepName === STEP_ASSOCIAR_DOCUMENTOS;
};
const _resolveParams = (icmWorkItem, ambiente) => {

    const serviceParams = {};
    console.log(icmWorkItem.attributes);
    serviceParams.taskStep = 'exit';
    serviceParams.workflowName = icmWorkItem.workflowName;
    serviceParams.repositoryID = ecm.model.desktop.getDefaultRepositoryId();
    serviceParams.caseFolderID = _isAssociarDocumentos(icmWorkItem)
        ? icmWorkItem.attributes.EDP_SubFolderID
        : icmWorkItem.caseFolderId;
    serviceParams.vsID = icmWorkItem.attributes.EDP_vsIdInitiatingDoc;
    serviceParams.nrUsersSeleccionados =
        icmWorkItem.attributes.EDP_nrUtilizadores;
    serviceParams.taskResponse = icmWorkItem.attributes.EDP_Responses;
    serviceParams.envPrefix = `${ambiente}_`;
    serviceParams.realmName = 'dc=edp,dc=pt';
    serviceParams.lockedUser = icmWorkItem.lockedUser;
    serviceParams.Empresa = icmWorkItem.attributes.EDP_Empresa;
    serviceParams.UO = icmWorkItem.attributes.EDP_UnidadeOrganizativa;
    serviceParams.PO = icmWorkItem.attributes.EDP_PosicaoOrganizativa;
    console.log('====CALLING VALIDATE WF DATA====', serviceParams);
    return serviceParams;
};

const _actionImprimir = EDP_IdDocPrincipal => {
    promiseRetrieveItem(EDP_IdDocPrincipal).then(item => {
        const imprimirAction = ecm.model.desktop.actionLists.ItemContextMenu.find(
            ({ id }) => id === 'EDPPrintStampedPDFAction'
        );
        const repository = ecm.model.desktop.getDefaultRepository();
        imprimirAction.performAction(repository, [item]);
    });
};
const _actionEncaminhar = (CaseFolderID, EDP_vsIdInitiatingDoc) => {
    const defaultRepository = ecm.model.desktop.getDefaultRepository();
    const utilizador = JSON.parse(sessionStorage['name']);
    window.eDPLaunchForwardWFAction(
        defaultRepository,
        defaultRepository.connectionPoint,
        CaseFolderID,
        EDP_vsIdInitiatingDoc,
        ecm.model.desktop.userId,
        utilizador.company.id,
        utilizador.organizationPosition.id,
        utilizador.organizationUnit.id,
        false,
        false,
    );
};
const _actionDivulgar = (CaseFolderID, EDP_vsIdInitiatingDoc) => {
    const defaultRepository = ecm.model.desktop.getDefaultRepository();
    const utilizador = JSON.parse(sessionStorage['name']);
    window.eDPlaunchExtDivulgationWFAction(
        defaultRepository,
        defaultRepository.connectionPoint,
        CaseFolderID,
        EDP_vsIdInitiatingDoc,
        ecm.model.desktop.userId,
        utilizador.company.id,
        utilizador.organizationPosition.id,
        utilizador.organizationUnit.id,
        false,
        false,
    );
};

function continueSubmitWF(icmWorkItem, response) {
    _promiseWFTaskValidation(icmWorkItem)
        .then(_onBehalfOf)
        .then(editable => {
            console.log('===COMPLETING STEP of editable=====', editable);
            editable.selectedResponse = response;
            editable.completeStep(
                a => {
                    switch (response) {
                        case selectedResponses.SUBMETER:
                            _showModal(icmWorkItem);
                            break;
                    }
                    console.log('Success', a);
                },
                a => console.log('error', a)
            );
        });
}

function definirMensagemEncaminhamento(
    assunto,
    workclass_name,
    EDP_Responses,
    stepName
) {
    console.log(
        '==== assinatura método definirMensagemEncaminhamento ====',
        assunto,
        workclass_name,
        EDP_Responses,
        stepName
    );
    if (
        workclass_name === 'EDP_EDP_wf_parecer_decisao' &&
        EDP_Responses === 'Rejeitar' &&
        stepName === 'Emitir Parecer'
    ) {
        return [
            `Encaminhado com sucesso`,
            `Encaminhamento: Pedido de Parecer – Devolução
            ${assunto}`
        ];
    }
    if (
        workclass_name === 'EDP_wf_parecer_decisao' &&
        EDP_Responses === 'Responder' &&
        stepName === 'Emitir Parecer'
    ) {
        return [
            `Encaminhado com sucesso`,
            `Encaminhamento: Parecer Emitido
            ${assunto}`
        ];
    }
    if (
        workclass_name === 'EDP_wf_parecer_decisao' &&
        EDP_Responses === 'Rejeitar ' &&
        stepName === 'Emitir Decisão'
    ) {
        return [
            `Encaminhado com sucesso`,
            `Encaminhamento: Pedido de Decisão – Devolução
            ${assunto}`
        ];
    }
    if (
        workclass_name === 'EDP_wf_parecer_decisao' &&
        EDP_Responses === 'Responder ' &&
        stepName === 'Emitir Decisão'
    ) {
        return [
            `Encaminhado com sucesso`,
            `Encaminhamento: Decisão Emitida
            ${assunto}`
        ];
    }
    // Using REGEX (multiple patterns), look here this awesome link about: https://tinyurl.com/y2f5fkae
    if (POSSIBLE_EDP_RESPONSES.ENCAMINHAMENTO_GENERICO.test(EDP_Responses)) {
        return [
            `Encaminhado com sucesso`,
            `Encaminhamento: Informação
            ${assunto}`
        ];
    }
}

function definirMensagemAlterarPropriedades(
    assunto,
    F_WorkClassname,
    EDP_isPowerUser,
    EDP_isDonoDocumento
) {
    console.log(
        'valores do metodo',
        assunto,
        F_WorkClassname,
        EDP_isPowerUser,
        EDP_isDonoDocumento
    );
    if (F_WorkClassname === 'EDP_wf_changeKeywords') {
        return [
            { title: true, message: `Operação realizada com sucesso` },
            `A alteração / adição de keywords foi concluída.`
        ];
    }
    if (F_WorkClassname === 'EDP_wf_changePhysicalLocation') {
        return [
            { title: true, message: `Operação realizada com sucesso` },
            `A alteração de localização física foi concluída.`
        ];
    }
    if (F_WorkClassname === 'EDP_wf_changeOtherProps' && !EDP_isPowerUser) {
        return [
            { title: true, message: `Operação realizada com sucesso` },
            `O pedido de alteração de propriedades foi enviado para validação.`
        ];
    }
    if (F_WorkClassname === 'EDP_wf_changeOtherProps' && EDP_isPowerUser) {
        return [
            { title: true, message: `Operação realizada com sucesso` },
            `A alteração de propriedades foi concluída.`
        ];
    }
    if (
        F_WorkClassname === 'EDP_wf_changeConfidentiality' &&
        !EDP_isDonoDocumento
    ) {
        return [
            { title: true, message: `Pedido enviado com sucesso` },
            `A alteração de confidencialidade foi solicitada ao dono do documento.`,
            `Caso seja aceite, esta alteração não terá impacto nos acessos já atribuídos na divulgação e/ou encaminhamento do documento em questão, `,
            `ou na sua utilização noutros processos documentos.`
        ];
    }
    if (
        F_WorkClassname === 'EDP_wf_changeConfidentiality' &&
        EDP_isDonoDocumento
    ) {
        return [
            { title: true, message: `Operação realizada com sucesso` },
            `A alteração do nível de confidencialidade foi concluída.`
        ];
    }
    return [{ title: true, message: `Enviado com sucesso` }, , assunto];
}

function nextSteps(
    stepName,
    EDP_IdDocPrincipal,
    CaseFolderID,
    EDP_vsIdInitiatingDoc,
    EDP_Responses
) {
    const stepImprimir = {
        key: '6',
        icon: 'IMPRESSORA',
        label: 'Descarregar para Impressão',
        action: () => _actionImprimir(EDP_IdDocPrincipal),
        performAction: true,
        inverseColor: true,
        shouldStay: true
    };

    const imprimir = deveImprimir(stepName, EDP_Responses)
        ? stepImprimir
        : undefined;

    return [
        {
            key: '4',
            icon: 'ENCAMINHAR',
            label: 'Encaminhar Documento(s)',
            action: () =>
                _actionEncaminhar(CaseFolderID, EDP_vsIdInitiatingDoc),
            performAction: true
        },
        {
            key: '5',
            icon: 'DIVULGAR',
            label: 'Divulgar(E-mail, Correio, Fax)',
            action: () => _actionDivulgar(CaseFolderID, EDP_vsIdInitiatingDoc),
            performAction: true
        },
        imprimir
    ];
}
function deveImprimir(stepName, EDP_Responses) {
    return (
        stepName === 'Carta' ||
        stepName === 'Fax' ||
        EDP_Responses === 'DivulgarFax' ||
        EDP_Responses === 'DivulgarCarta'
    );
}
