var BundleAnalyzerPlugin = require('webpack-bundle-analyzer').BundleAnalyzerPlugin;
var path = require('path');
var webpack = require('webpack');

module.exports = {
  entry: './src/index.js',
  output: {
    path: path.resolve(__dirname, 'build'),
    filename: 'index.js',
    libraryTarget: 'commonjs2' // THIS IS THE MOST IMPORTANT LINE! :mindblow: I wasted more than 2 days until realize this was the line most important in all this guide.
	},
	performance: {
    hints: false
	},
  module: {
    rules: [
      {
        test: /\.js$/,
        include: path.resolve(__dirname, 'src'),
        exclude: /(bower_components|build)/,
        use: {
          loader: 'babel-loader?presets[]=@babel/es2015!ts-loader',
          options: {
            presets: ['env', 'react']
          }
        }
      },
      {
        test: /\.css$/,
        include: path.resolve(__dirname, 'src'),
        exclude: /(bower_components|build)/,
        use: [
          { loader: 'style-loader' },
          {
            loader: 'css-loader',
            options: {
              modules: false
            }
          }
        ]
      },
      {
        test: /\.svg$/,
        loader: 'file-loader',
        query: {
          name: 'static/media/[name].[hash:8].[ext]'
        }
      }
    ],
    loaders: []
  },
  plugins: [
    // Minify the code.
    new webpack.optimize.UglifyJsPlugin({
      test: /\.js($|\?)/i,
      sourceMap: true,
      uglifyOptions: {
        compress: {
          screw_ie8: true, // React doesn't support IE8
					warnings: false,
					pure_getters: true,
					unsafe: true,
					unsafe_comps: true,
					conditionals: true,
					unused: true,
					comparisons: true,
					sequences: true,
					dead_code: true,
					evaluate: true,
					if_return: true,
					join_vars: true
        },
        mangle: {
          screw_ie8: true
        },
        output: {
          comments: false,
          screw_ie8: true
				},
				exclude: [/\.min\.js$/gi]
      }
		}),
		
		// https://medium.com/@tkssharma/optimizing-bundle-size-webpack-ab4efdfc1d15
		// new BundleAnalyzerPlugin({
		// 	analyzerMode: 'static'
		// }),

		new webpack.optimize.AggressiveMergingPlugin(),

		new webpack.DefinePlugin({
			'process.env.NODE_ENV': JSON.stringify('production')
		})

  ],

  externals: {
    react: 'commonjs react' // this line is just to use the React dependency of our parent-testing-project instead of using our own React.
  }
};
